==> trayplugin1.1.zip: trayplugin1.1/source/trayplugin.c <==
/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2002 Hiroyuki Yamamoto and the Sylpheed-Claws Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif
#include "defs.h"

#include "plugin.h"
#include "prefs_gtk.h"
#include "intl.h"
#include "common/utils.h"

#include "tray.h"

/* ------------------------------------------------------------------------- */

static gint findmain_timeout_tag=0;
static struct TrayPluginPage *trayplugin_page;

static PrefParam param[] = {
	{"enable", "TRUE", &trayplugin_enable, P_BOOL,
	 NULL, NULL, NULL},
	{"persistent", "TRUE", &trayplugin_persistent, P_BOOL,
	 NULL, NULL, NULL},
	{"windowstate", "0", &trayplugin_windowstate, P_INT,
	 NULL, NULL, NULL},
	{"laststate", "0", &trayplugin_laststate, P_INT,
	 NULL, NULL, NULL},

	{NULL, NULL, NULL, P_OTHER, NULL, NULL, NULL}
};

struct TrayPluginPage
{
	PrefsPage page;
	
	GtkWidget *enable;
	GtkWidget *persistent;
	GtkWidget *state_none;
	GtkWidget *state_restore;
	GtkWidget *state_minimized;
	GtkWidget *state_maximized;
};

/* -------------------------------------------------------------------------
 * Wait for MainWindow to be shown
 */

static gint findmain_timeout_cb(gpointer *data) {
	gboolean initialized;
	mainwindow = mainwindow_get_mainwindow();
	g_return_val_if_fail(mainwindow, TRUE);

	initialized = tray_init(mainwindow->window);
	if (trayplugin_windowstate == TRAY_STATE_RESTORE)
		SetWindowState(trayplugin_laststate);
	else
		SetWindowState(trayplugin_windowstate);
	return FALSE; /* remove timeout */
}

/* -------------------------------------------------------------------------
 * Save plugin settings to sylpheedrc
 */

void trayplugin_save_config()
{
	PrefFile *pfile;
	gchar *rcpath;

	debug_print("Saving Tray plugin Page\n");
	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, COMMON_RC, NULL);
	pfile = prefs_write_open(rcpath);
	g_free(rcpath);
	if (!pfile || (prefs_set_block_label(pfile, "TrayPlugin") < 0))
		return;

	if (prefs_write_param(param, pfile->fp) < 0) {
		g_warning("failed to write Tray plugin configuration to file\n");
		prefs_file_close_revert(pfile);
		return;
	}
	prefs_file_close(pfile);
}

/* -------------------------------------------------------------------------
 * Settings window (create, destroy, save)
 */

static void trayplugin_create_widget_func(PrefsPage * _page, GtkWindow *window, gpointer data)
{
	struct TrayPluginPage *page = (struct TrayPluginPage *) _page;

	GtkWidget *table1;
	GtkWidget *persistent;
	GtkWidget *enable;
	GtkWidget *frame2;
	GtkWidget *vbox2;
	GSList *vbox2_group = NULL;
	GtkWidget *state_none;
	GtkWidget *state_restore;
	GtkWidget *state_minimized;
	GtkWidget *state_maximized;

	table1 = gtk_table_new (4, 1, FALSE);
	gtk_widget_ref (table1);
	gtk_widget_show (table1);

	persistent = gtk_check_button_new_with_label (_("Persistent (always show icon)"));
	gtk_widget_ref (persistent);

	gtk_widget_show (persistent);
	gtk_table_attach (GTK_TABLE (table1), persistent, 0, 1, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	enable = gtk_check_button_new_with_label (_("Enable Tray icon"));
	gtk_widget_ref (enable);

	gtk_widget_show (enable);
	gtk_table_attach (GTK_TABLE (table1), enable, 0, 1, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	frame2 = gtk_frame_new (_("Window state after startup"));
	gtk_widget_ref (frame2);

	gtk_widget_show (frame2);
	gtk_table_attach (GTK_TABLE (table1), frame2, 0, 1, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 16, 4);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox2);

	gtk_widget_show (vbox2);
	gtk_container_add (GTK_CONTAINER (frame2), vbox2);

	state_none = gtk_radio_button_new_with_label (vbox2_group, _("Do not set"));
	vbox2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (state_none));
	gtk_widget_ref (state_none);

	gtk_widget_show (state_none);
	gtk_box_pack_start (GTK_BOX (vbox2), state_none, FALSE, FALSE, 0);

	state_restore = gtk_radio_button_new_with_label (vbox2_group, _("Restore last setting"));
	vbox2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (state_restore));
	gtk_widget_ref (state_restore);

	gtk_widget_show (state_restore);
	gtk_box_pack_start (GTK_BOX (vbox2), state_restore, FALSE, FALSE, 0);

	state_minimized = gtk_radio_button_new_with_label (vbox2_group, _("Start minimized"));
	vbox2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (state_minimized));
	gtk_widget_ref (state_minimized);

	gtk_widget_show (state_minimized);
	gtk_box_pack_start (GTK_BOX (vbox2), state_minimized, FALSE, FALSE, 0);

	state_maximized = gtk_radio_button_new_with_label (vbox2_group, _("Start maximized"));
	vbox2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (state_maximized));
	gtk_widget_ref (state_maximized);

	gtk_widget_show (state_maximized);
	gtk_box_pack_start (GTK_BOX (vbox2), state_maximized, FALSE, FALSE, 0);


	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable), trayplugin_enable);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(persistent), trayplugin_persistent);
	switch (trayplugin_windowstate) {
	case TRAY_STATE_NONE:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(state_none), TRUE);
		break;
	case TRAY_STATE_RESTORE: 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(state_restore), TRUE);
		break;
	case TRAY_STATE_MINIMIZED: 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(state_minimized), TRUE);
		break;
	case TRAY_STATE_MAXIMIZED: 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(state_maximized), TRUE);
		break;
	}

	page->enable		= enable;
	page->persistent	= persistent;
	page->state_none	= state_none;
	page->state_restore	= state_restore;
	page->state_minimized	= state_minimized;
	page->state_maximized	= state_maximized;
	page->page.widget	= table1;
}

static void trayplugin_destroy_widget_func(PrefsPage *_page)
{
	debug_print("Destroying Tray plugin widget\n");
}

static void trayplugin_save_func(PrefsPage *_page)
{
	struct TrayPluginPage *page = (struct TrayPluginPage *) _page;

	debug_print("Saving Tray plugin Page\n");
	trayplugin_enable	= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->enable));
	trayplugin_persistent	= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->persistent));
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->state_none)))
		trayplugin_windowstate = TRAY_STATE_NONE;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->state_restore)))
		trayplugin_windowstate = TRAY_STATE_RESTORE;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->state_minimized)))
		trayplugin_windowstate = TRAY_STATE_MINIMIZED;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->state_maximized)))
		trayplugin_windowstate = TRAY_STATE_MAXIMIZED;

	trayplugin_save_config();
	ShowTrayIcon(trayplugin_enable & trayplugin_persistent);
}

static void trayplugin_destroy_func(PrefsPage *_page)
{
	debug_print("Destroying Tray plugin Page\n");
	g_free(_page);
}

/* -------------------------------------------------------------------------
 * Exported plugin functions (init, destroy, identify)
 */

gint plugin_init(gchar **error)
{
	struct TrayPluginPage *page;
	trayplugin_persistent = TRUE;
	trayplugin_enable = TRUE;

	/* init config */
	prefs_set_default(param);
	prefs_read_config(param, "TrayPlugin", COMMON_RC);

	page = g_new0(struct TrayPluginPage, 1);
	page->page.path = _("Filtering/TrayPlugin");
	page->page.create_widget = trayplugin_create_widget_func;
	page->page.destroy_widget = trayplugin_destroy_widget_func;
	page->page.save_page = trayplugin_save_func;
	page->page.destroy_page = trayplugin_destroy_func;
	prefs_gtk_register_page((PrefsPage *) page);
	trayplugin_page = page;
	debug_print("Tray plugin config page created\n");

	/* wait for mainwindow to show up */
	findmain_timeout_tag = gtk_timeout_add( 100, findmain_timeout_cb, NULL );
	debug_print("Tray plugin loaded\n");
	return 0;
}

void plugin_done()
{
	gtk_timeout_remove( findmain_timeout_tag );
	tray_close();
	if (trayplugin_windowstate == TRAY_STATE_RESTORE) {
		HWND hwnd = GDK_DRAWABLE_XID(mainwindow->window->window);
		if (IsIconic(hwnd) || !IsWindowVisible(hwnd))
			trayplugin_laststate = TRAY_STATE_MINIMIZED;
		else if (IsZoomed(hwnd))
			trayplugin_laststate = TRAY_STATE_MAXIMIZED;
		else	trayplugin_laststate = TRAY_STATE_NONE;
		trayplugin_save_config();
	}
	debug_print("Tray plugin unloaded\n");
}

const gchar *plugin_name()
{
	return "Tray plugin 1.1";
}

const gchar *plugin_desc()
{
	return "This Plugin minimizes Sylpheed to tray.\n";
}

==> trayplugin1.2.zip: trayplugin1.2/source/trayplugin.c <==
/*
 * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
 * Copyright (C) 1999-2002 Hiroyuki Yamamoto and the Sylpheed-Claws Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif
#include "defs.h"

#include "plugin.h"
#include "prefs_gtk.h"
#include "intl.h"
#include "common/utils.h"

#include "tray.h"

/* ------------------------------------------------------------------------- */

static gint findmain_timeout_tag=0;
static struct TrayPluginPage *trayplugin_page;

static PrefParam param[] = {
	{"enable", "TRUE", &trayplugin_enable, P_BOOL,
	 NULL, NULL, NULL},
	{"persistent", "TRUE", &trayplugin_persistent, P_BOOL,
	 NULL, NULL, NULL},
	{"windowstate", "0", &trayplugin_windowstate, P_INT,
	 NULL, NULL, NULL},
	{"laststate", "0", &trayplugin_laststate, P_INT,
	 NULL, NULL, NULL},

	{NULL, NULL, NULL, P_OTHER, NULL, NULL, NULL}
};

struct TrayPluginPage
{
	PrefsPage page;
	
	GtkWidget *enable;
	GtkWidget *persistent;
	GtkWidget *state_none;
	GtkWidget *state_restore;
	GtkWidget *state_minimized;
	GtkWidget *state_maximized;
};

/* -------------------------------------------------------------------------
 * Wait for MainWindow to be shown
 */

static gint findmain_timeout_cb(gpointer *data) {
	gboolean initialized;
	mainwindow = mainwindow_get_mainwindow();
	g_return_val_if_fail(mainwindow, TRUE);

	initialized = tray_init(mainwindow->window);
	if (trayplugin_windowstate == TRAY_STATE_RESTORE)
		SetWindowState(trayplugin_laststate);
	else
		SetWindowState(trayplugin_windowstate);
	return FALSE; /* remove timeout */
}

/* -------------------------------------------------------------------------
 * Save plugin settings to sylpheedrc
 */

void trayplugin_save_config()
{
	PrefFile *pfile;
	gchar *rcpath;

	debug_print("Saving Tray plugin Page\n");
	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, COMMON_RC, NULL);
	pfile = prefs_write_open(rcpath);
	g_free(rcpath);
	if (!pfile || (prefs_set_block_label(pfile, "TrayPlugin") < 0))
		return;

	if (prefs_write_param(param, pfile->fp) < 0) {
		g_warning("failed to write Tray plugin configuration to file\n");
		prefs_file_close_revert(pfile);
		return;
	}
	prefs_file_close(pfile);
}

/* -------------------------------------------------------------------------
 * Settings window (create, destroy, save)
 */

static void trayplugin_create_widget_func(PrefsPage * _page, GtkWindow *window, gpointer data)
{
	struct TrayPluginPage *page = (struct TrayPluginPage *) _page;

	GtkWidget *table1;
	GtkWidget *persistent;
	GtkWidget *enable;
	GtkWidget *frame2;
	GtkWidget *vbox2;
	GSList *vbox2_group = NULL;
	GtkWidget *state_none;
	GtkWidget *state_restore;
	GtkWidget *state_minimized;
	GtkWidget *state_maximized;
GtkWidget *label1;

	table1 = gtk_table_new (4, 1, FALSE);
	gtk_widget_ref (table1);
	gtk_widget_show (table1);

	persistent = gtk_check_button_new_with_label (_("Persistent (always show icon)"));
	gtk_widget_ref (persistent);

	gtk_widget_show (persistent);
	gtk_table_attach (GTK_TABLE (table1), persistent, 0, 1, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	enable = gtk_check_button_new_with_label (_("Enable Tray icon"));
	gtk_widget_ref (enable);

	gtk_widget_show (enable);
	gtk_table_attach (GTK_TABLE (table1), enable, 0, 1, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

label1 = gtk_label_new("Window state after startup:");
gtk_widget_show (label1);
	gtk_table_attach (GTK_TABLE (table1), label1, 0, 1, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label1), 4, 8);

	frame2 = gtk_frame_new (NULL); //_("Window state after startup"));
	gtk_widget_ref (frame2);

	gtk_widget_show (frame2);
	gtk_table_attach (GTK_TABLE (table1), frame2, 0, 1, 3, 4,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 16, 0);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox2);

	gtk_widget_show (vbox2);
	gtk_container_add (GTK_CONTAINER (frame2), vbox2);

	state_none = gtk_radio_button_new_with_label (vbox2_group, _("No change"));
	vbox2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (state_none));
	gtk_widget_ref (state_none);

	gtk_widget_show (state_none);
	gtk_box_pack_start (GTK_BOX (vbox2), state_none, FALSE, FALSE, 0);

	state_restore = gtk_radio_button_new_with_label (vbox2_group, _("Restore last setting"));
	vbox2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (state_restore));
	gtk_widget_ref (state_restore);

	gtk_widget_show (state_restore);
	gtk_box_pack_start (GTK_BOX (vbox2), state_restore, FALSE, FALSE, 0);

	state_minimized = gtk_radio_button_new_with_label (vbox2_group, _("Start minimized"));
	vbox2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (state_minimized));
	gtk_widget_ref (state_minimized);

	gtk_widget_show (state_minimized);
	gtk_box_pack_start (GTK_BOX (vbox2), state_minimized, FALSE, FALSE, 0);

	state_maximized = gtk_radio_button_new_with_label (vbox2_group, _("Start maximized"));
	vbox2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (state_maximized));
	gtk_widget_ref (state_maximized);

	gtk_widget_show (state_maximized);
	gtk_box_pack_start (GTK_BOX (vbox2), state_maximized, FALSE, FALSE, 0);


	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable), trayplugin_enable);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(persistent), trayplugin_persistent);
	switch (trayplugin_windowstate) {
	case TRAY_STATE_NONE:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(state_none), TRUE);
		break;
	case TRAY_STATE_RESTORE: 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(state_restore), TRUE);
		break;
	case TRAY_STATE_MINIMIZED: 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(state_minimized), TRUE);
		break;
	case TRAY_STATE_MAXIMIZED: 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(state_maximized), TRUE);
		break;
	}

	page->enable		= enable;
	page->persistent	= persistent;
	page->state_none	= state_none;
	page->state_restore	= state_restore;
	page->state_minimized	= state_minimized;
	page->state_maximized	= state_maximized;
	page->page.widget	= table1;
}

static void trayplugin_destroy_widget_func(PrefsPage *_page)
{
	debug_print("Destroying Tray plugin widget\n");
}

static void trayplugin_save_func(PrefsPage *_page)
{
	struct TrayPluginPage *page = (struct TrayPluginPage *) _page;

	debug_print("Saving Tray plugin Page\n");
	trayplugin_enable	= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->enable));
	trayplugin_persistent	= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->persistent));
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->state_none)))
		trayplugin_windowstate = TRAY_STATE_NONE;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->state_restore)))
		trayplugin_windowstate = TRAY_STATE_RESTORE;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->state_minimized)))
		trayplugin_windowstate = TRAY_STATE_MINIMIZED;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->state_maximized)))
		trayplugin_windowstate = TRAY_STATE_MAXIMIZED;

	trayplugin_save_config();
	ShowTrayIcon(trayplugin_enable & trayplugin_persistent);
}

static void trayplugin_destroy_func(PrefsPage *_page)
{
	debug_print("Destroying Tray plugin Page\n");
	g_free(_page);
}

/* -------------------------------------------------------------------------
 * Exported plugin functions (init, destroy, identify)
 */

gint plugin_init(gchar **error)
{
	struct TrayPluginPage *page;
	trayplugin_persistent = TRUE;
	trayplugin_enable = TRUE;

	/* init config */
	prefs_set_default(param);
	prefs_read_config(param, "TrayPlugin", COMMON_RC);

	page = g_new0(struct TrayPluginPage, 1);
	page->page.path = _("Plugins/TrayPlugin");
	page->page.create_widget = trayplugin_create_widget_func;
	page->page.destroy_widget = trayplugin_destroy_widget_func;
	page->page.save_page = trayplugin_save_func;
	page->page.destroy_page = trayplugin_destroy_func;
	prefs_gtk_register_page((PrefsPage *) page);
	trayplugin_page = page;
	debug_print("Tray plugin config page created\n");

	/* wait for mainwindow to show up */
	findmain_timeout_tag = gtk_timeout_add( 100, findmain_timeout_cb, NULL );
	debug_print("Tray plugin loaded\n");
	return 0;
}

void plugin_done()
{
	gtk_timeout_remove( findmain_timeout_tag );
	tray_close();
	if (trayplugin_windowstate == TRAY_STATE_RESTORE) {
		HWND hwnd = GDK_DRAWABLE_XID(mainwindow->window->window);
		if (IsIconic(hwnd) || !IsWindowVisible(hwnd))
			trayplugin_laststate = TRAY_STATE_MINIMIZED;
		else if (IsZoomed(hwnd))
			trayplugin_laststate = TRAY_STATE_MAXIMIZED;
		else	trayplugin_laststate = TRAY_STATE_NONE;
		trayplugin_save_config();
	}
	debug_print("Tray plugin unloaded\n");
}

const gchar *plugin_name()
{
#ifdef _DEBUG
	return "Tray plugin 1.2 (debug)";
#else
	return "Tray plugin 1.2";
#endif
}

const gchar *plugin_desc()
{
	return  "Minimizes Sylpheed to system tray and restores "
		"the last window state (minimized/maximized).";
}
